"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var ProstoreSsrContext_1 = require("./ProstoreSsrContext");
var prostore_1 = require("@proscom/prostore");
var useConnectStore_1 = require("./useConnectStore");
var ProstoreContext_1 = require("./ProstoreContext");
/**
 * Хук, подключающий компонент к стору и запрашивающий данные
 * в соответствии с variables
 */
function useRequestStore(storeOrName, variables, options) {
    if (options === void 0) { options = null; }
    // Подключаем стор из контекста
    var store = ProstoreContext_1.useContextStore(storeOrName);
    // Подключение к контексту, который хранит все результаты
    var ssrContext = react_1.useContext(ProstoreSsrContext_1.ProstoreSsrContext);
    // Фейковое состояние для ререндера
    var _a = react_1.useState(0), rerender = _a[1];
    // Локальная копия состояния стора
    var stateRef = react_1.useRef(null);
    // Ссылка на текущий стор
    var storeRef = react_1.useRef(null);
    // Если стор изменился
    if (store !== storeRef.current) {
        // Запоминаем текущий стор
        storeRef.current = store;
        // Если в контексте есть значение, используем его
        stateRef.current = store.getValueFromContext(ssrContext);
    }
    // Колбек для обновления состояния
    var setState = react_1.useCallback(function (newState) {
        if (stateRef.current !== newState) {
            stateRef.current = newState;
            rerender(function (s) { return s + 1; });
        }
    }, []);
    // Подписываемся на изменения стора
    useConnectStore_1.useConnectStore(store, setState);
    // Локальная копия состояния стора
    var state = stateRef.current;
    // Проверка, нужно ли запрашивать данные
    var queryCheck = prostore_1.checkRequestState(state, variables);
    // Мемоизированный колбек для загрузки данных
    var loadData = react_1.useCallback(function () {
        return store
            .loadData(variables, __assign({ ssr: ssrContext && ssrContext.isServer }, options))
            .then(function () {
            store.setContextValue(ssrContext);
        });
    }, [store, variables, options]);
    // Запрос данных, если он нужен
    react_1.useEffect(function () {
        if (queryCheck.request) {
            // Вероятность ошибки в этом месте мала, т.к.
            // основные ошибки обрабатываются в RequestStore
            // Поэтому мы ее просто залогируем
            loadData().catch(function (e) { return console.error(e); });
        }
    }, [queryCheck, loadData]);
    // Если мы сейчас в режиме серверного рендеринга, то запрашиваем данные
    if (ssrContext && ssrContext.isServer && queryCheck.request && store.ssrId) {
        ssrContext.registerPromise(loadData());
    }
    // Возвращаем то, что может быть полезно в самом компоненте
    return { check: queryCheck, state: state, load: loadData };
}
exports.useRequestStore = useRequestStore;
function useRequestStoreVars(storeOrName, variableCreator, args) {
    var variables = react_1.useMemo(variableCreator, args);
    return useRequestStore(storeOrName, variables);
}
exports.useRequestStoreVars = useRequestStoreVars;
//# sourceMappingURL=useRequestStore.js.map