import { useContext, createContext } from 'react';
import { IStore } from '@proscom/prostore';

export type IProstoreContext = {
  [name: string]: IStore<any>;
};

export type StoreOrName<State> = string | IStore<State>;

export const ProstoreContext = createContext<IProstoreContext>(null);

export function useContextStore<State>(
  storeOrName: StoreOrName<State>
): IStore<State> {
  const context = useContext(ProstoreContext);

  if (typeof storeOrName === 'string') {
    const store = context[storeOrName];
    if (!store) {
      throw new Error(
        `Store '${storeOrName}' is not registered in the context`
      );
    }
    return store;
  }

  return storeOrName;
}
