import { createContext } from 'react';
import { IProstoreSsrContext } from '@proscom/prostore';

export const ProstoreSsrContext = createContext<ProstoreSsrManager>(null);

export type IStates = {
  [name: string]: any;
};

export interface IProstoreSsrManagerParams {
  isServer?: boolean;
  initialStates?: IStates;
}

export class ProstoreSsrManager implements IProstoreSsrContext {
  public isServer: boolean = false;
  public states: IStates = {};
  public promises: Set<Promise<any>> = new Set();
  public disabled: boolean = false;

  constructor({ isServer, initialStates }: IProstoreSsrManagerParams) {
    this.isServer = isServer || false;
    this.states = initialStates || this.states;
  }

  registerPromise(promise: Promise<any>) {
    this.promises.add(promise);
    promise.then(() => {
      this.promises.delete(promise);
    });
  }

  hasPromises(): boolean {
    return this.promises.size > 0;
  }

  waitForCurrentPromises(): Promise<any> {
    return Promise.all(this.promises);
  }

  getState(key: string): any {
    if (this.disabled) return null;
    return this.states[key];
  }

  setState(key: string, value: any) {
    if (this.disabled) return;
    this.states[key] = value;
  }

  getStates(): IStates {
    return this.states;
  }

  setDisabled(value: boolean) {
    this.disabled = value;
  }
}
