import { BehaviorSubject, Observable } from 'rxjs';
import { useEffect, useState } from 'react';

/**
 * Подписка на обзервабл
 */
export function useObservable<State>(
  obs$: Observable<State>,
  setState: (state: State) => any
) {
  useEffect(() => {
    const sub = obs$.subscribe((state: State) => {
      setState(state);
    });
    return () => {
      sub.unsubscribe();
    };
  }, [obs$, setState]);
}

/**
 * Подписка на обзервабл с локальным стейтом
 */
export function useObservableState<State>(
  obs$: Observable<State>,
  initialState: State = null
): State {
  const [state, setState] = useState(initialState);
  useObservable(obs$, setState);
  return state;
}

/**
 * Подписка на BehaviorSubject
 */
export function useSubject<State>(sub$: BehaviorSubject<State>) {
  return useObservableState(sub$, sub$.value);
}
