import { useMemo, useState } from 'react';
import { StoreOrName, useContextStore } from './ProstoreContext';
import { useConnectStore } from './useConnectStore';
import { IStore } from '@proscom/prostore';

/**
 * Хук, подключающий компонент к стору
 */
export function useStore<State>(
  storeOrName: StoreOrName<State>
): [State, IStore<State>] {
  const store = useContextStore(storeOrName);
  const [state, setState] = useState(store.state);
  useConnectStore(store, setState);
  return [state, store];
}

/**
 * Хук, подключающий компонент к стору
 */
export function useStoreState<State>(storeOrName: StoreOrName<State>): State {
  const store = useContextStore(storeOrName);
  const [state, setState] = useState(store.state);
  useConnectStore(store, setState);
  return state;
}

/**
 * Хук, подключащий компонент к стору и возвращающий часть состояния
 */
export function useStoreSelect<State, Derived>(
  storeOrName: StoreOrName<State>,
  select: (state: State) => Derived
): [Derived, IStore<State>] {
  const [state, store] = useStore(storeOrName);
  const selectedState = useMemo(() => select(state), [select, state]);
  return [selectedState, store];
}
